/*******************************************************************************
Demo project for Olimex board: MOD-LCD 4.3" RevA with LPC 1788
For IAR 6.21

In order to work correctly you must connect:

0)  USB serial cables in the respective UEXT pins as follow:
    2-GND (blue wire); 3-TX (green wire); 4-RX (red wire).
    UART must be configured:
    BaudRate: 115200
    DataBits: 8
    StopBits: 1
    Parity:   None

1)  SD card formated or containing some files/folders to the slot.

2)  USB-A Mini to the USB-OTG connector so you can browse the SDCard via
    Windows Explorer.

3)  JLink to JTag connector if you intend to program or debug it.
    Otherwise if you wish to use the project standalone you don't need the JLink.

Brief description:
On Power-ON/Reset on the console terminal will be shown a welcome message.
After that there will be something similar to a command prompt where you can
send several commands. You can see the list of command if you send command:
  'help all'
For more details of a specific command type help <cmd>
For example: 'help files'

As for the rest of the peripherals:
When you push the button the yellow LED is ON and otherwise the LED is OFF.
When you touch the screen on the console will be shown the coordinates
of the point where you touch it (if it's only one position, otherwise the
results will be incorrect).

Author:   Stanimir Petev
Company:  OLIMEX
Date:     18.09.2012
*******************************************************************************/

#include "includes.h"
#include <yfuns.h>

#define TIMER0_TICK_PER_SEC   2
#define UPDATE_SHOW_DLY       1
#define DLY_100US             500

#pragma location="USB_DMA_RAM"
#pragma data_alignment=4
__no_init Int8U Lun0Buffer[512];

// SPP +
#pragma section = "SDRAM"
#pragma location = "SDRAM"
EmbeddedFileSystem efs;
#pragma section = "SDRAM"
#pragma location = "SDRAM"
EmbeddedFile file_r, file_w;
#pragma section = "SDRAM"
#pragma location = "SDRAM"
DirList MyList;
char GlobalPath[MAX_PATH_LENGTH] = "/";   // root dir
int PicIndex=0, PicCount, SDCardError = 0, BaudRateKey=3, Touch_Threshold=5;

pPic_t Displayed_Picture = NULL;
pPic_t *Picture;

pPic_t Load_Bitmap_to_Memory (char Name[]);
void Load_All_Bitmap_to_Memory (void);
// SPP -


volatile Int32U Tick;

extern FontType_t Terminal_6_8_6;
extern FontType_t Terminal_9_12_6;
extern FontType_t Terminal_18_24_12;

/** external data **/
#pragma section=".intvec"
/** public functions **/
/*************************************************************************
 * Function Name: InitClock
 * Parameters: void
 * Return: void
 *
  * Description: Initialize PLL0 and clocks' dividers. PLL = 96MHz,
 *               CPU - 96MHz, PCLK - 48 MHz
 *
 *************************************************************************/
void InitClock(void)
{
  /*Sys Clock Select as CPU clock
    divider 1:1*/
  CLK_SetCpuClk(CPUSEL_CLKSYS,1);
  /*Select IRC oscilator as Sys clock*/
  CLK_SetSysClk(SYSSEL_IRCOSC);
  /*Enable Main oscilator*/
  CLK_MainOscSet(CLK_ENABLE,MOSCRNG_1_20MHZ);
  /*Select Main oscilator as Sys clock*/
  CLK_SetSysClk(SYSSEL_MOSC);
  /*Enable set PLL 96Hz (M = 8; P = 1)*/
  CLK_SetMainPll(CLK_ENABLE, 8-1, 0);
  /*Set peripheral divider 1:2. Peripheral clock 48MHz*/
  CLK_SetPeriphClk(2);
  /*PLL0 out is used as the input to the CPU clock divider
  divider 1:1. CPU runs at 96MHz*/
  CLK_SetCpuClk(CPUSEL_CLKPLL,1); 
}

/*************************************************************************
 * Function Name: __low_level_init
 * Parameters: void
 * Return: 
 *
 * Description: Low level system init (clock, flash accelerator, 
 *              SDRAM and vector table address)
 *              
 *
 *************************************************************************/
int __low_level_init (void)
{
  /*if debug is in SDRAM
    clock and SDRAM initialization is made
    in mac file*/
#ifndef SDRAM_DEBUG 
  int cpuclk;
  /* Flash accelerator save value*/
  FLASHCFG = (6UL<<12) | 0x3AUL;
  /*Clock Init*/
  InitClock();
  /*Flash accelerator init*/
  cpuclk = CLK_GetClock(CLK_CPU);
  
  if( cpuclk < 20000000 ){
    FLASHCFG = (0x0UL<<12) | 0x3AUL;
  } else if( cpuclk < 40000000 ){
    FLASHCFG = (0x1UL<<12) | 0x3AUL;
  } else if( cpuclk< 60000000 ){
    FLASHCFG = (0x2UL<<12) | 0x3AUL;
  } else if( cpuclk < 80000000 ){
    FLASHCFG = (0x3UL<<12) | 0x3AUL;
  } else if( cpuclk < 100000000 ){
    FLASHCFG = (0x4UL<<12) | 0x3AUL;
  }
  /*SDRAM init*/
  SDRAM_Init();
#endif
  /*Set vector table location*/
  VTOR  = (unsigned int)__segment_begin(".intvec");

  return  1;
}

/*************************************************************************
 * Function Name: TMR0_IRQHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Timer 0 interrupt handler
 *
 *************************************************************************/
void TMR0_IRQHandler (void)
{
  ++Tick;
  /*clear interrupt*/
  T0IR_bit.MR0INT = 1;
  /*Dummy read*/
  T0IR;
}

/*************************************************************************
 * Function Name: TMR0_Init
 * Parameters: Int32U tps - Ticks per second
 *
 * Return: none
 *
 * Description: Timer 0 start
 *
 *************************************************************************/
void TMR0_Init(Int32U tps)
{
  Tick = 0;
  /* Enable TIM0 clock*/
  PCONP_bit.PCTIM0 = 1;

  /*Init Timer 0 */
  T0TCR_bit.CE = 0;      /* counting  disable */
  T0TCR_bit.CR = 1;      /* set reset */
  T0TCR_bit.CR = 0;      /* release reset */
  T0CTCR_bit.CTM = 0;    /* Timer Mode: every rising PCLK edge */
  T0MCR_bit.MR0I = 1;    /* Enable Interrupt on MR0*/
  T0MCR_bit.MR0R = 1;    /* Enable reset on MR0 */
  T0MCR_bit.MR0S = 0;    /* Disable stop on MR0 */
  /*set timer 0 period*/
  T0PR = 0;
  T0MR0 = CLK_GetClock(CLK_PERIPH)/(tps);
  /* init timer 0 interrupt */
  T0IR_bit.MR0INT = 1;  /* clear pending interrupt */
  T0TCR_bit.CE = 1;     /* counting Enable */
  /*Enable NVIC TMR0 Interrupt*/
  NVIC_IntEnable(NVIC_TIMER0);
}

/*************************************************************************
 * Function Name: TMR0_Stop
 * Parameters: none
 *
 * Return: none
 *
 * Description: Timer 0 stop
 *
 *************************************************************************/
void TMR0_Stop(void)
{
  /* counting  disable */
  T0TCR_bit.CE = 0;
  /* Enable NVIC TMR0 Interrupt */
  NVIC_IntDisable(NVIC_TIMER0);
  /* Timer 0 Clock disable */
  PCONP_bit.PCTIM0 = 0;
}

/*************************************************************************
 * Function Name: DelayResolution100us
 * Parameters: Int32U Dly
 * Return: void
 *
 * Description: Delay Dly * [100us]
 *		
 *************************************************************************/
void DelayResolution100us(Int32U Dly)
{
  for(; Dly; Dly--)
  {
    for(volatile Int32U j = DLY_100US; j; j--)
    {
    }
  }
}


/*************************************************************************
 * Function Name: GetClock
 * Parameters: clocks_t clock - 
 * Return: Int32U
 *
 * Description: return clock in [Hz]
 *
 *************************************************************************/
Int32U GetClock(clocks_t clock)
{
Int32U freq = 0;
  
  switch(clock)
  {
  case sysclk:
      if(CLKSRCSEL_bit.CLKSRC)
      {/*Main Oscilator*/
        freq = MAIN_OSC_FREQ;
      }
      else
      {/*Internal oscilator*/
        freq = I_RC_OSC_FREQ;
      }
    break;   
  case pll_clk:
    if(PLL0STAT_bit.PLLE_STAT)
    {/*PLL0 is enabled*/
      freq = GetClock(sysclk)*(PLL0STAT_bit.MSEL+1);
    }
    break;
  case alt_pll_clk:
    if(PLL1STAT_bit.PLLE_STAT)
    {/*PLL1 is enabled*/
      freq = MAIN_OSC_FREQ*(PLL1STAT_bit.MSEL+1);
    }
    break;
  case cclk:
    if(CCLKSEL_bit.CCLKSEL)
    {/*it is pll clock*/
      freq = GetClock(pll_clk);
    }
    else
    {/*it is sys clock*/
      freq = GetClock(sysclk);
    }
    freq /= CCLKSEL_bit.CCLKDIV;
    break;
  case pclk:
    if(PCLKSEL_bit.PCLKDIV)
    {/*periphera devider is enabled*/
      if(CCLKSEL_bit.CCLKSEL)
      {/*it is pll clock*/
        freq = GetClock(pll_clk);
      }
      else
      {/*it is sys clock*/
        freq = GetClock(sysclk);
      }
      freq /= PCLKSEL_bit.PCLKDIV;
    }
    break;
  case emc_clk:
    freq = GetClock(cclk);
    if(EMCCLKSEL_bit.EMCDIV)
    {/*emc is at half of cpu freqency*/
      freq /= 2;
    }
    break;
  case usb_clk:
    if(USBCLKSEL_bit.USBDIV)
    {/*usb devider is enabled*/
      switch(USBCLKSEL_bit.USBSEL)
      {
      case 0:
        freq = GetClock(sysclk);
        break;
      case 1:
        freq = GetClock(pll_clk);
        break;
      case 2:
        freq = GetClock(alt_pll_clk);
        break;
      }
      
      freq /= USBCLKSEL_bit.USBDIV;
    }
    break;
  case spifi_clk:
    if(SPIFICLKSEL_bit.SPIFIDIV)
    {/*spifi devider is enabled*/
      switch(SPIFICLKSEL_bit.SPIFISEL)
      {
      case 0:
        freq = GetClock(sysclk);
        break;
      case 1:
        freq = GetClock(pll_clk);
        break;
      case 2:
        freq = GetClock(alt_pll_clk);
        break;
      }
      
      freq /= SPIFICLKSEL_bit.SPIFIDIV;
    break;
    }
  }
  
  return freq;
}

void Initialize (void)
{
  /*Disable interrupts*/
  __disable_interrupt();
    
  /*Init LCD Controller*/
  /*Disable LCD controller*/
  GLCD_Ctrl (FALSE);
  /*Init LCD and copy picture in video RAM*/
//  GLCD_Init (Automn.pPicStream, NULL);
  GLCD_Init (NULL, NULL);
  /*Disable & init cursor*/
  GLCD_Cursor_Dis(0);
  GLCD_Copy_Cursor ((Int32U *)Cursor, 0, sizeof(Cursor)/sizeof(Int32U));
  GLCD_Cursor_Cfg(CRSR_FRAME_SYNC | CRSR_PIX_32);
  GLCD_Move_Cursor(C_GLCD_H_SIZE/2, C_GLCD_V_SIZE/2);
  GLCD_Cursor_En(0);
  
  /*Configure USB Clock*/
  CLK_SetUsbClk(USBSEL_CLKPLL,USB_CLK_DIV);

  // Init touch screen
  TouchScrInit();
  
  // UART Init
  UartInit(UART_0,4,NORM);
  // Configurate Uart 0
  UART_Configurate (115200);
  
  /* Init SCSI module */
  ScsiInit();
  
  /*Enable interrupts*/
  __enable_interrupt();
  
    // LUNs Init
  LunInit(SD_DISK_LUN,SdDiskInit,SdGetDiskCtrlBkl,SdDiskIO);

  // Init SD card driver
  // SCSI init push init SD driver message for an each LUN
  for(Int32U i = 0; i < SCSI_LUN_NUMB; i++)
    // Implement LUNs messages
    if(LunImp(i))

  /*Start timer 0*/ 
  TMR0_Init(TIMER0_TICK_PER_SEC);

  /* Soft connection enable */
  USB_ConnectRes(TRUE);
  
  // Init SD card file system
  if (efs_init (&efs, 0))   // Init File system
  {
    SDCardError = 1;
//    Error();
  }
  if (ls_openDir (&MyList, &efs.myFs, GlobalPath))   // Init root dir
  {
    SDCardError = 1;
//    Error();
  }
  
  // initialize button
  BUT_FDIR = BUT_FDIR & (~BUT_MASK);
  
  /* LED init */
  USB_OTG_LINK_LED_IOCON = 0x20;
  USB_OTG_LINK_LED_FDIR = USB_OTG_LINK_LED_FDIR | USB_OTG_LINK_LED_MASK;
  
  Synchronize_Buffers ();
  
  /*Enable LCD*/
  GLCD_Ctrl (TRUE);
}

void Mass_Storage_Routine (void)
{
    for(Int32U i = 0; i < SCSI_LUN_NUMB; i++)
      /* Implement LUNs messages */
      LunImp(i);
    if(Tick)
    {
      Tick = 0;
      /* Update MMC/SD card status */
      SdStatusUpdate();
    }
}

void Touch_Screen_Routine (void)
{
  static int cursor_x, cursor_y;
  static ToushRes_t XY_Touch;
  static int TouchScreenFlag=0;

  // Touch screen routine
  if(TouchGet(&XY_Touch))
  {
      if( abs((int)XY_Touch.X - cursor_x) > Touch_Threshold || abs((int)XY_Touch.Y - cursor_y) > Touch_Threshold)
      {
        char Output[32];
        cursor_x = XY_Touch.X;
        cursor_y = XY_Touch.Y;
        if (TouchScreenFlag)
          Write ("Drag to position: ");
        else
        {
          NewLine ();
          Write ("Screen touched on position: ");
        }
        sprintf (Output, "X = %d; Y = %d", cursor_x, cursor_y);
        WriteLn (Output);
        GLCD_Move_Cursor(cursor_x, cursor_y);
      }
      TouchScreenFlag = 1;
  }
  else
  {
    if (TouchScreenFlag)
    {
      Write ("Screen untouched!\n\r");
      PrintGlobalPath ();
      TouchScreenFlag = 0;
    }
  }
}

void UART_Routine (void)
{
  static char UART_Input[256];
  if (ReadLn (UART_Input))
  {
    int i, argc;      // arguments count
    char **argv=NULL; // arguments values
    CommandKey Cmd;   // command type

    Cmd = Form_Command (UART_Input, &argc, &argv);
    Dispatch_Command (Cmd, argc, argv);

    // free memory for the arguments
    for (i=0; i<argc; i++)
      free (argv[i]);
    free (argv);
    PrintGlobalPath ();
    
  }
}

/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/
int main(void)
{
  char Welcome_Message[] =  "\f"\
                            "*********************************\n\r"\
                            "*   OLIMEX - MOD LCD 4.3 demo   *\n\r"\
                            "* Use the console to manipulate *\n\r"\
                            "*   the pictures with figures   *\n\r"\
                            "*     type 'help all' to see    *\n\r"\
                            "*        all the options        *\n\r"\
                            "*    Note: The Console isn't    *\n\r"\
                            "*         case sensitive        *\n\r"\
                            "*********************************\n\r";
  char SDCardProblem_Message[] = "SD card problem detected!\n\r";
  Initialize ();
  
  Write (Welcome_Message);
  if (SDCardError)
    Write (SDCardProblem_Message);
  PrintGlobalPath ();
   
  while(1)
  {
    UART_Routine ();
    Mass_Storage_Routine ();
    Touch_Screen_Routine ();    
    
    if (Button_is_Pressed())
      LED_State (ON);
    else
      LED_State (OFF);
  }
}
/** private functions **/
